#include "Api/OsaDef.h"
#include "../OsaRtl.h"
#include "../OsaMemory.h"
#include "../OsaMemDbg.h"

#define EXTRA_DPRINTF 1      // set to 1 for more debug printouts
       
#ifdef new
#undef new
#endif

#if DBG
/******************************************************************************
******************************************************************************/
void * operator new(size_t cbSize, char *szFileName, int nLine)
{
  return ::operator new(cbSize, OSA_MEMORY_NONPAGED, szFileName, nLine);
}


/******************************************************************************
******************************************************************************/
void * operator new[](size_t cbSize, char *szFileName, int nLine)
{
  return ::operator new(cbSize, OSA_MEMORY_NONPAGED, szFileName, nLine);
}


/******************************************************************************
******************************************************************************/
void * operator new(size_t cbSize, OsaMemoryPoolType type, char *szFileName, int nLine)
{
  OSA_ASSERT(cbSize);  // Normally it is possible to allocate 0 bytes in C++ but not with ExAllocatePool()
  void *ptr;
  int bAtomic;

  if (OsaGetCurrentExeLevel() == OSA_EXE_LEVEL_CONTROL) {
    // At this execution level it is OK to sleep and wait for a page (i.e. use OSALINUX_GFP_KERNEL mode)
    bAtomic = OSALINUX_GFP_KERNEL;
  } else {
    // At an elevated execution level we should not go sleep. I.e. we can not wait for a page (i.e. use OSALINUX_GFP_ATOMIC mode)
    bAtomic = OSALINUX_GFP_ATOMIC;
  }
  
  ptr = osalinux_kmalloc(cbSize + OSA_EXTRA_MEM_NEEDED, bAtomic);

  if (ptr) {

    ptr = OsaMakeMemoryHeaderTail(ptr, cbSize, szFileName, nLine);

    if (type == OSA_MEMORY_PAGED) {
      OSA_DPRINTF(("OsaLinux: Memory is Paged!\n"));
    }

  } else {
    (*__newhandler)(cbSize);  
  }

  OSA_ASSERT(ptr);  // Since we might not always check memory allocation return values, make sure that we know when memory allocation failed
  return ptr;
}

#endif


/******************************************************************************
******************************************************************************/
void *operator new(size_t cbSize)
{
  return ::operator new(cbSize, OSA_MEMORY_NONPAGED);
}


/******************************************************************************
******************************************************************************/
void *operator new[](size_t cbSize)
{
  return ::operator new(cbSize, OSA_MEMORY_NONPAGED);
}


/******************************************************************************
******************************************************************************/
void *operator new(size_t cbSize, OsaMemoryPoolType type)
{

  OSA_ASSERT(cbSize);  // Normally it is possible to allocate 0 bytes in C++ but not with ExAllocatePool()
  void *ptr;
  int bAtomic;

  if (OsaGetCurrentExeLevel() == OSA_EXE_LEVEL_CONTROL) {
    // At this execution level it is OK to sleep and wait for a page (i.e. use OSALINUX_GFP_KERNEL mode)
    bAtomic = OSALINUX_GFP_KERNEL;
  } else {
    // At an elevated execution level we should not go sleep. I.e. we can not wait for a page (i.e. use OSALINUX_GFP_ATOMIC mode)
    bAtomic = OSALINUX_GFP_ATOMIC;
  }

  ptr = osalinux_kmalloc(cbSize + OSA_EXTRA_MEM_NEEDED, bAtomic);

  if (ptr) {

#if DBG
    ptr = OsaMakeMemoryHeaderTail(ptr, cbSize, "Unknown Name", 0);

    if (type == OSA_MEMORY_PAGED) {
      OSA_DPRINTF(("OsaLinux: Memory is Paged!\n"));
    }
#endif

  } else {
    (*__newhandler)(cbSize);  
  }

  OSA_ASSERT(ptr);  // Since we might not always check memory allocation return values, make sure that we know when memory allocation failed
  return ptr;
}


/******************************************************************************
******************************************************************************/
void operator delete(void *ptr)
{
  //OSA_DPRINTF(("OsaLinux: Deleting ptr = %p\n", ptr));
  //OSA_ASSERT(NULL != p);

#if DBG
  if (ptr) {
    void *pUser = ptr;   // Goofy assignments back and forth solely to take the speed hit in the DBG version only
    ptr = OsaUndoMemoryHeaderTail(pUser);
  }
#endif

  if (ptr) {
    osalinux_kfree(ptr);
  }
}

/******************************************************************************
******************************************************************************/
void operator delete[](void *ptr)
{
  //OSA_DPRINTF(("OsaLinux: Ptr = %p\n", ptr));
  //OSA_ASSERT(NULL != p);

  delete ptr;
}

